<?php

namespace App\Http\Controllers;

use App\Constants\Status;
use App\Events\MessageSend;
use App\Lib\CurlRequest;
use App\Models\CronJob;
use App\Models\CronJobLog;
use App\Models\Device;
use App\Models\Sms;
use Carbon\Carbon;
use Exception;

class CronController extends Controller
{
    public function cron()
    {
        $general            = gs();
        $general->last_cron = now();
        $general->save();

        $crons = CronJob::with('schedule');

        if (request()->alias) {
            $crons->where('alias', request()->alias);
        } else {
            $crons->where('next_run', '<', now())->where('is_running', Status::YES);
        }
        $crons = $crons->get();
        foreach ($crons as $cron) {
            $cronLog              = new CronJobLog();
            $cronLog->cron_job_id = $cron->id;
            $cronLog->start_at    = now();
            if ($cron->is_default) {
                $controller = new $cron->action[0];
                try {
                    $method = $cron->action[1];
                    $controller->$method();
                } catch (\Exception $e) {
                    $cronLog->error = $e->getMessage();
                }
            } else {
                try {
                    CurlRequest::curlContent($cron->url);
                } catch (\Exception $e) {
                    $cronLog->error = $e->getMessage();
                }
            }
            $cron->last_run = now();
            $cron->next_run = now()->addSeconds($cron->schedule->interval);
            $cron->save();

            $cronLog->end_at = $cron->last_run;

            $startTime         = Carbon::parse($cronLog->start_at);
            $endTime           = Carbon::parse($cronLog->end_at);
            $diffInSeconds     = $startTime->diffInSeconds($endTime);
            $cronLog->duration = $diffInSeconds;
            $cronLog->save();
        }
        if (request()->target == 'all') {
            $notify[] = ['success', 'Cron executed successfully'];
            return back()->withNotify($notify);
        }
        if (request()->alias) {
            $notify[] = ['success', keyToTitle(request()->alias) . ' executed successfully'];
            return back()->withNotify($notify);
        }
    }

    public function sendSMS()
    {
        return $this->sms(0);
    }

    public function resendSMS()
    {
        return $this->sms(1);
    }

    protected function sms($et)
    {
        try {
            $currentTime = now()->format('Y-m-d h:i');

            $gs = gs();
            try {
                $sms = Sms::whereIn('status', [Status::SMS_INITIAL, Status::SMS_SCHEDULED])
                    ->where('sms_type', Status::SMS_TYPE_SEND)
                    ->orderBy('id', 'ASC')
                    ->where('et', $et)
                    ->whereHas('device', function ($q) {
                        $q->where('status', Status::ENABLE);
                    })
                    ->select('id', 'message', 'device_slot_number', 'mobile_number', 'device_id')
                    ->take(15)
                    ->get();

                foreach ($sms->groupBy('device_id') as $k => $newSMS) {
                    event(new MessageSend([
                        'success'       => true,
                        'original_data' =>
                        [
                            'message'   => $newSMS->toArray(),
                            'device_id' => Device::where('id', $k)->first()->device_id,
                        ],
                    ]));
                }
                if ($et == 0) {
                    Sms::where('status', Status::DISABLE)->where('schedule', '<=', $currentTime)
                        ->where('sms_type', Status::SMS_TYPE_SEND)
                        ->whereHas('device', function ($q) {
                            $q->where('status', Status::ENABLE);
                        })
                        ->orderBy('id', 'ASC')
                        ->take(15)
                        ->where('et', 0)
                        ->update(['et' => 1]);
                }

                $gs->cron_error_message = null;
            } catch (Exception $ex) {
                $gs->cron_error_message = $ex->getMessage();
                info($ex->getMessage());
            }

            $gs->last_cron = now();
            $gs->save();

            return "EXECUTED";
        } catch (\Throwable $th) {
            throw new \Exception($th->getMessage());
        }
    }

}
